@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -211,7 +211,7 @@ SQLRETURN SQLBrowseConnect(
     SQLRETURN ret;
     SQLCHAR s1[ 100 + LOG_MESSAGE_LEN ], s2[ 100 + LOG_MESSAGE_LEN];
 	SQLWCHAR *uc_in_str;
-    int warnings;
+    int warnings = 0;
 
     /*
      * check connection
@@ -269,7 +269,7 @@ SQLRETURN SQLBrowseConnect(
                 ERROR_08002, NULL, 
                 connection -> environment -> requested_version );
 
-        return function_return( -1, connection, SQL_ERROR );
+        return function_return( IGNORE_THREAD, connection, SQL_ERROR );
     }
 
     thread_protect( SQL_HANDLE_DBC, connection );
@@ -393,6 +393,8 @@ SQLRETURN SQLBrowseConnect(
 
         if ( !__connect_part_one( connection, lib_name, driver_name, &warnings ))
         {
+            __disconnect_part_four( connection );       /* release unicode handles */
+
             dm_log_write( __FILE__,
                     __LINE__,
                     LOG_INFO,
@@ -411,6 +413,7 @@ SQLRETURN SQLBrowseConnect(
                         "Error: IM001" );
 
             __disconnect_part_one( connection );
+            __disconnect_part_four( connection );       /* release unicode handles */
             __post_internal_error( &connection -> error,
                     ERROR_IM001, NULL,
                     connection -> environment -> requested_version );
@@ -470,6 +473,7 @@ SQLRETURN SQLBrowseConnect(
                       "Error: IM001" );
 
         __disconnect_part_one( connection );
+        __disconnect_part_four( connection );       /* release unicode handles */
         __post_internal_error( &connection -> error,
                                ERROR_IM001, NULL,
                                connection -> environment -> requested_version );
@@ -555,6 +559,7 @@ SQLRETURN SQLBrowseConnect(
     	if ( ret != SQL_NEED_DATA ) 
 		{
         	__disconnect_part_one( connection );
+            __disconnect_part_four( connection );       /* release unicode handles */
         	connection -> state = STATE_C2;
 		}
 		else 
@@ -572,13 +577,14 @@ SQLRETURN SQLBrowseConnect(
 
         if( ret == SQL_SUCCESS_WITH_INFO )
         {
-            function_return_ex( SQL_HANDLE_DBC, connection, ret, TRUE );
+            function_return_ex( IGNORE_THREAD, connection, ret, TRUE );
         }
 
         if ( !__connect_part_two( connection ))
         {
             __disconnect_part_two( connection );
             __disconnect_part_one( connection );
+            __disconnect_part_four( connection );       /* release unicode handles */
             if ( log_info.log_flag )
             {
                 sprintf( connection -> msg, 
